@@ -8,8 +8,9 @@
 
 static std::shared_ptr<spdlog::logger> logger_ = nullptr;
 
-// See eg https://spdlog.docsforge.com/v1.x/3.custom-formatting/#pattern-flags
-const std::string default_log_pattern = "%^[%Y-%m-%d %H:%M:%S.%e] [%n] [Process: %P] [%l] %v%$";
+// See eg https://spdlog.docsforge.com/v1.x/3.custom-formatting/#pattern-flags or
+// https://github.com/gabime/spdlog/wiki/3.-Custom-formatting
+const std::string default_log_pattern = "[%Y-%m-%d %H:%M:%S.%e] [%n] [Process: %P] [%l] %v";
 
 //' R Accessor Functions for spdlog Logger
 //'
@@ -37,6 +38,9 @@ const std::string default_log_pattern = "%^[%Y-%m-%d %H:%M:%S.%e] [%n] [Process:
 //' \sQuote{critical}.  A message issued below the current threshold is not displayed whereas
 //' a message at or above the current threshold is displayed.  The default level is \sQuote{warn}.
 //'
+//' @seealso The logging pattern format is described in at the repo in the page
+//' \url{https://github.com/gabime/spdlog/wiki/3.-Custom-formatting}.
+//'
 //' @param name A character variable with the logging instance name, default value is \sQuote{default}.
 //' @param level A character variable with the default logging level, default value is \sQuote{warn}.
 //' @param s A character variable with the logging pattern, level or message.
@@ -46,7 +50,7 @@ const std::string default_log_pattern = "%^[%Y-%m-%d %H:%M:%S.%e] [%n] [Process:
 //' @return Nothing is returned from these functions as they are invoked for their side-effects.
 //'
 //' @examples
-//' log_setup("demo")
+//' log_setup("demo")  # at default level 'warn'
 //' log_info("this message is NOT seen")
 //' log_set_level("debug")
 //' log_info("this message is seen")
@@ -71,6 +75,12 @@ void log_setup(const std::string& name = "default", const std::string& level = "
     spdlog::set_level(spdlog::level::from_str(level));
 }
 
+//' @rdname log_setup
+// [[Rcpp::export]]
+void log_init(const std::string& level = "warn") {
+    log_setup("r", level);
+}
+
 //' @rdname log_setup
 // [[Rcpp::export]]
 void log_filesetup(const std::string& filename, const std::string &name = "default", const std::string& level = "warn") {
@@ -158,3 +168,54 @@ void log_critical(const std::string& s) {
     assert_and_setup_if_needed();
     spdlog::critical(s);
 }
+
+template <typename T> Rcpp::XPtr<T> make_xptr(T* p) {
+    return Rcpp::XPtr<T>(p);
+}
+
+//' R Accessor Functions for spdlog Stopwatch
+//'
+//' A set of functions provides access to the \code{spdlog} stopwatch facilty. As \code{stopwatch}
+//' object is a simple container around a C++ \code{std::chrono} object which (essentially) reports
+//' elapsed-time since creation. The object is exported to R via an external pointer permitting use
+//' from both R and C++.
+//'
+//' Several functions are provided:
+//' \describe{
+//'    \item{\code{get_stopwatch}}{Returns a stopwatch object (as an S3 object).}
+//'    \item{\code{elapsed_stopwatch}}{Returns elapsed time for stopwatch in seconds.}
+//'    \item{\code{format_stopwatch}}{Returns elapsed time for stopwatch as character variable.}
+//' }
+//' The \code{stopwatch} object has \code{print} and \code{format} methods.
+//'
+//' @param sw An S3 object of type \code{stopwatch}.
+//' @param x An S3 object of type \code{stopwatch}.
+//' @param ... Dotted argument required by generic, unused here.
+//'
+//' @return The desired object is returned: respectively, a stopwatch object as an external pointer
+//' in an S3 class, the elapsed time in seconds as a double, or formatted as a character variable.
+//'
+//' @examples
+//' w <- get_stopwatch()
+//' Sys.sleep(0.2)
+//' elapsed_stopwatch(w)
+//' format_stopwatch(w)
+//' @rdname get_stopwatch
+// [[Rcpp::export]]
+Rcpp::XPtr<spdlog::stopwatch> get_stopwatch() {
+    auto sw = make_xptr(new spdlog::stopwatch());
+    sw.attr("class") = Rcpp::CharacterVector::create("stopwatch", "externalptr");
+    return sw;
+}
+
+//' @rdname get_stopwatch
+// [[Rcpp::export]]
+double elapsed_stopwatch(Rcpp::XPtr<spdlog::stopwatch> sw) {
+    return sw->elapsed().count();
+}
+
+//' @rdname get_stopwatch
+// [[Rcpp::export]]
+std::string format_stopwatch(Rcpp::XPtr<spdlog::stopwatch> sw) {
+    return std::to_string(elapsed_stopwatch(sw));
+}
